function [nVAC, tVAC, dVAC] = vacuum(opt, f, sigAC, fDC, Pdc)
% [nVAC, tVAC, dVAC] = vacuum(opt, f, sigAC, fDC, Pdc)
%
% vacuum.m produces noise plots from vacuum. Injected vacuum is defined as
% as VN in "setup.m".
%
% nVAC[frequency, vacuum number, PD number] : noise for each vacuum
% tVAC[frequency, lenght DOF] : total noise for each length DOF
% dVAC[frequency, PD number] : total noise for each PD
%
% Unit of all graphs is the same definition as Thomas's tool, i.e. the raw
% noise unit [W/rHz] is converted to the unit which the amplitude of input
% vacuum is defined to be 1, [1/rHz]
%
% Rresult is verified with Thomas's tool for DCreadout port, but results
% for other RF readout are not correct due to no RF vacuum injection. This
% problem will be fixed in the next version. Current version asuumes that
% RF power makes flat shotnoise.

global rd

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%% Initial setting
setup;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%%% Noise from vacuum
% nVAC: raw vacuum noise for each injection, [W/rHz]
% tVAC: total vacuum noise, [W/rHz]
% plots: the same definition as Thomas's tool, i.e. the amplitude of input
% vacuum is 1, [1/rHz]

  dVAC = zeros(length(f), opt.Nprobe);
  for pn = 1:length(PN)
    for vn = 1:length(VN)
      if isempty(findstr('IM', opt.probe(PN(pn)).name))
        if isempty(findstr('HD', opt.probe(PN(pn)).name))
          aVAC1 = abs(squeeze(sigAC(PN(pn), VN(vn)  , :))) * qeff ;
          aVAC2 = abs(squeeze(sigAC(PN(pn), VN(vn)+1, :))) * qeff;
          nVAC(:, vn, PN(pn)) = sqrt(aVAC1.^2 + aVAC2.^2);
          oVAC(:, vn, pn) = nVAC(:, vn, PN(pn));
          dVAC(:, PN(pn)) = sqrt(dVAC(:, PN(pn)).^2 + nVAC(:, vn, PN(pn)).^2);
        else
          %for HD
          aVAC1a = abs(squeeze(sigAC(PN(pn), VN(vn), :))) * qeff;
          aVAC2a = abs(squeeze(sigAC(PN(pn), VN(vn)+1, :))) * qeff;
          aVAC1b = abs(squeeze(sigAC(PN(pn)+1, VN(vn), :))) * qeff;
          aVAC2b = abs(squeeze(sigAC(PN(pn)+1, VN(vn)+1, :))) * qeff;

          % each noise: nVAC
          nVAC(:, vn, PN(pn)) = sqrt(aVAC1a.^2 + aVAC2a.^2);
          nVAC(:, vn, PN(pn)+1) = sqrt(aVAC1b.^2 + aVAC2b.^2);  
          % total noise: dVAC
          dVAC(:, PN(pn)) = sqrt(dVAC(:, PN(pn)).^2 + (nVAC(:, vn, PN(pn))).^2);
          dVAC(:, PN(pn)+1) = sqrt(dVAC(:, PN(pn)+1).^2 + (nVAC(:, vn, PN(pn)+1)).^2);

          oVAC(:, vn, pn) = (nVAC(:, vn, PN(pn)) + nVAC(:, vn, PN(pn)+1));
        end
      else
        %for DDM         
        aVAC1M = abs(squeeze(sigAC(PN(pn), VN(vn), :))) * qeff;
        aVAC2M = abs(squeeze(sigAC(PN(pn), VN(vn)+1, :))) * qeff;
        aVAC1P = abs(squeeze(sigAC(PN(pn)+2, VN(vn), :))) * qeff;
        aVAC2P = abs(squeeze(sigAC(PN(pn)+2, VN(vn)+1, :))) * qeff;

        % each noise: nVAC
        nVAC(:, vn, PN(pn)) = sqrt(aVAC1P.^2 + aVAC2P.^2);
        nVAC(:, vn, PN(pn)+2) = sqrt(aVAC1M.^2 + aVAC2M.^2);  
        % total noise: dVAC
        dVAC(:, PN(pn)) = sqrt(dVAC(:, PN(pn)).^2 + (nVAC(:, vn, PN(pn))).^2);
        dVAC(:, PN(pn)+2) = sqrt(dVAC(:, PN(pn)+2).^2 + (nVAC(:, vn, PN(pn)+2)).^2);

        % Coherently sum: oVAC for MI and PI
        oVAC(:, vn, pn) = (nVAC(:, vn, PN(pn)) + nVAC(:, vn, PN(pn)+2)) / 2;
      end
    end

    % Add flat shot noise for not detected RF (or carrier), might be wrong
    if isempty(findstr('IM', opt.probe(PN(pn)).name))
      if isempty(findstr('HD', opt.probe(PN(pn)).name))
        if isempty(findstr('DC', opt.probe(PN(pn)).name))
          %for single demodulation         
%          oVAC(:, vn+1, pn) = sqrt(abs(Phv * (Pdc(pn) - abs(fDC(FN(pn), nCA))^2 * qeff))); % flat shotnoise due to RF sidebands
          oVAC(:, vn+1, pn) = sqrt(abs(Phv * Pdc(pn)) * qeff); % flat shotnoise due to RF sidebands
          dVAC(:, PN(pn)) = sqrt(dVAC(:, PN(pn)).^2 + oVAC(:, vn+1, pn).^2);
          tVAC(:, pn) = dVAC(:, PN(pn));
        else
          %for DC readout         
          oVAC(:, vn+1, pn) = sqrt(abs(Phv * (Pdc(pn) - abs(fDC(FN(pn), nCA))^2)) * qeff); % flat shotnoise due to RF sidebands
          dVAC(:, PN(pn)) = sqrt(dVAC(:, PN(pn)).^2 + oVAC(:, vn+1, pn).^2);
          tVAC(:, pn) = dVAC(:, PN(pn));
        end
      else
        %for Homodyne detection         
        oVAC(:, vn+1, pn) = sqrt(abs(Phv * (Pdc(pn) - abs(fDC(FN(pn), nCA))^2)) * qeff); % flat shotnoise due to RF sidebands
        dVAC(:, PN(pn))   = sqrt(dVAC(:, PN(pn)  ).^2 + oVAC(:, vn+1, pn).^2);
        dVAC(:, PN(pn)+1) = sqrt(dVAC(:, PN(pn)+1).^2 + oVAC(:, vn+1, pn).^2);
        tVAC(:, pn) = dVAC(:, PN(pn)) + dVAC(:, PN(pn)+1);
      end
    else
      %for DDM         
      oVAC(:, vn+1, pn) = sqrt(abs(Phv * Pdc(pn)) * qeff); % flat shotnoise due to carrier and RF sidebands power, 2 is for two PDs
      dVAC(:, PN(pn)  ) = sqrt(dVAC(:, PN(pn)  ).^2 + oVAC(:, vn+1, pn).^2);
      dVAC(:, PN(pn)+2) = sqrt(dVAC(:, PN(pn)+2).^2 + oVAC(:, vn+1, pn).^2);
      tVAC(:, pn) = (dVAC(:, PN(pn)) + dVAC(:, PN(pn)+2)) / 2; % Coherently sum
    end    

    % figures
    figure(400 + pn)
    if MeterUnitOn == 1
      loglog(f, [oVAC(:, :, pn), tVAC(:, pn)] / sqrt(abs(Phv * Pdc(pn))));
      ylabel('1/rtHz')
    else
      loglog(f, [oVAC(:, :, pn), tVAC(:, pn)])
      ylabel('W/rtHz')
    end
    title(['Noise from vacuum at ' char(ND(pn))])
    legend('laser', 'dark', 'POXin', 'POXout', 'POYin', 'POYout',...
           'REFL Attenuator', 'AS Attenuator', 'OMC Attenuator', 'POX Attenuator',...
           'RF flat shotnoise', 'total',3)
    xlabel('Hz')
    grid on
    h = gcf;
    for ns = 1 : length(ext)
      saveas(h, [rd '/' num2str(400 + pn) '_noise_' char(NC(pn))  '.' char(ext(ns))]);
    end

    %figure(410 + pn)
    %loglog(f, [nVAC(:, :, pn), tVAC(:, pn)]);
    %title(['Raw noise from vacuum at ' char(ND(pn))])
    %legend('laser', 'dark', 'POXin', 'POXout', 'POYin', 'POYout', 'total',3)
    %xlabel('Hz')
    %ylabel('W/rtHz')
    %grid on
    %h = gcf;
    %saveas(h, [rd '/' num2str(410 + pn) '.' ext]);
  end